﻿using System;
using TwinCAT.Ads;
using TwinCAT.PlcOpen;
using TwinCAT.TypeSystem;

namespace BeckhoffAdsTester5
{
    class Program
    {
        static void Main(string[] args)
        {
            //实例化客户端类，连接AmsNetId以及851端口，并创建变量
            AdsClient client = new AdsClient();
            client.Connect(AmsNetId.Local, 851);
            uint handleTime = 0;
            uint handleDate = 0;

            try
            {
                //将创建的变量指向PLC变量
                handleTime = client.CreateVariableHandle("MAIN.t_value"); // TIME
                handleDate = client.CreateVariableHandle("MAIN.d_value"); // DATE

                //给予读取的数据大小，最大的PLC Open类型的数据为8个字节大小
                byte[] readBuffer = new byte[8]; 
                byte[] writeBuffer = new byte[8];

                // 读取时间
                client.Read(handleTime, readBuffer.AsMemory(0, TIME.MarshalSize));

                // 新建一个变量plcTime，处理读取的数据并赋值给plcTime
                TIME plcTime = null;
                PrimitiveTypeMarshaler.Default.Unmarshal(readBuffer.AsSpan(0, TIME.MarshalSize), out plcTime);
                TimeSpan time = plcTime.Time;
                Console.WriteLine("读取到的时间为：{0}", plcTime.Time);

                // 写入时间
                PrimitiveTypeMarshaler.Default.Marshal(time, writeBuffer.AsSpan());
                client.Write(handleTime, writeBuffer.AsMemory(0, TIME.MarshalSize));

                

                // 读取日期
                DATE plcDate = null;
                client.Read(handleDate, readBuffer.AsMemory(0, DATE.MarshalSize));
                

                // 新建一个变量dateTime，处理读取的数据并赋值给dateTime
                PrimitiveTypeMarshaler.Default.Unmarshal(readBuffer.AsSpan(0, DATE.MarshalSize), out plcDate);
                DateTimeOffset dateTime = plcDate.Date;
                Console.WriteLine("日期为：{0}", plcDate);

                // 写入日期
                PrimitiveTypeMarshaler.Default.Marshal(plcDate, writeBuffer.AsSpan());
                client.Write(handleDate, writeBuffer.AsMemory(0, DATE.MarshalSize));
            }
            finally
            {
                //释放缓存
                client.DeleteVariableHandle(handleTime);
                client.DeleteVariableHandle(handleDate);
            }
        }
    }
    
}
